@@ -138,6 +138,12 @@ const field_t	inode_core_flds[] = {
 	{ "extszinherit", FLDT_UINT1,
 	  OI(COFF(flags) + bitsz(__uint16_t) - XFS_DIFLAG_EXTSZINHERIT_BIT-1),C1,
 	  0, TYP_NONE },
+	{ "nodefrag", FLDT_UINT1,
+	  OI(COFF(flags) + bitsz(__uint16_t) - XFS_DIFLAG_NODEFRAG_BIT-1),C1,
+	  0, TYP_NONE },
+	{ "filestream", FLDT_UINT1,
+	  OI(COFF(flags) + bitsz(__uint16_t) - XFS_DIFLAG_FILESTREAM_BIT-1),C1,
+	  0, TYP_NONE },
 	{ "gen", FLDT_UINT32D, OI(COFF(gen)), C1, 0, TYP_NONE },
 	{ NULL }
 };
