@@ -4,12 +4,17 @@
 #
 
 AUTO=false
-while getopts ":aA" c
+while getopts ":aAy" c
 do
 	case $c in
-	a|A)	AUTO=true;;
+	a|A|y)	AUTO=true;;
 	esac
 done
+eval DEV=\${$#}
+if [ ! -e $DEV ]; then
+	echo "$0: $DEV does not exist"
+	exit 8
+fi
 if $AUTO; then
 	echo "$0: XFS file system."
 else
