@@ -91,6 +91,9 @@ static int
 init_check_command(
 	const cmdinfo_t	*ct)
 {
+	if (ct->flags & CMD_FLAG_GLOBAL)
+		return 1;
+
 	if (!file && !(ct->flags & CMD_NOFILE_OK)) {
 		fprintf(stderr, _("no files are open, try 'help open'\n"));
 		return 0;
