@@ -116,12 +116,17 @@ projects_free_space_data(
 	__uint64_t		*rused,
 	__uint64_t		*rfree)
 {
+	fs_quota_stat_t		qfs;
 	fs_disk_quota_t		d;
 	struct fsxattr		fsx;
 	uint			type = XFS_PROJ_QUOTA;
 	char			*dev = path->fs_name;
 	int			fd;
 
+	if (xfsquotactl(XFS_GETQSTAT, dev, type, 0, &qfs) < 0 ||
+			!(qfs.qs_flags & XFS_QUOTA_PDQ_ACCT))
+		return 0;
+
 	if ((fd = open(path->fs_dir, O_RDONLY)) < 0) {
 		fprintf(stderr, "%s: cannot open %s: %s\n",
 			progname, path->fs_dir, strerror(errno));
