@@ -26,7 +26,6 @@
 #include "dinode.h"
 #include "dir.h"
 #include "bmap.h"
-#include "prefetch.h"
 
 #if XFS_DIR_LEAF_MAPSIZE >= XFS_ATTR_LEAF_MAPSIZE
 #define XR_DA_LEAF_MAPSIZE	XFS_DIR_LEAF_MAPSIZE
@@ -781,9 +780,6 @@ traverse_int_dablock(xfs_mount_t	*mp,
 	node = NULL;
 	da_cursor->active = 0;
 
-	if (do_prefetch && (whichfork == XFS_DATA_FORK))
-		prefetch_dir1(mp, bno, da_cursor);
-
 	do {
 		/*
 		 * read in each block along the way and set up cursor
@@ -1952,12 +1948,6 @@ _("entry #%d, bno %d in directory %llu references group quota inode %llu\n"),
 				_("\twould clear ino number in entry %d...\n"),
 					i);
 			}
-		} else if (lino == old_orphanage_ino)  {
-			/*
-			 * do nothing, silently ignore it, entry has
-			 * already been marked TBD since old_orphanage_ino
-			 * is set non-zero.
-			 */
 		} else if ((irec_p = find_inode_rec(
 				XFS_INO_TO_AGNO(mp, lino),
 				XFS_INO_TO_AGINO(mp, lino))) != NULL)  {
