@@ -26,6 +26,7 @@
 #include "dir2.h"
 #include "bmap.h"
 #include "prefetch.h"
+#include "progress.h"
 
 /*
  * Tag bad directory entries with this.
@@ -102,11 +103,21 @@ da_read_buf(
 		bplist = bparray;
 	}
 	for (i = 0; i < nex; i++) {
+#ifdef XR_PF_TRACE
+		pftrace("about to read off %llu (len = %d)",
+			(long long)XFS_FSB_TO_DADDR(mp, bmp[i].startblock),
+			XFS_FSB_TO_BB(mp, bmp[i].blockcount));
+#endif
 		bplist[i] = libxfs_readbuf(mp->m_dev,
 				XFS_FSB_TO_DADDR(mp, bmp[i].startblock),
 				XFS_FSB_TO_BB(mp, bmp[i].blockcount), 0);
 		if (!bplist[i])
 			goto failed;
+#ifdef XR_PF_TRACE
+		pftrace("readbuf %p (%llu, %d)", bplist[i],
+			(long long)XFS_BUF_ADDR(bplist[i]),
+			XFS_BUF_COUNT(bplist[i]));
+#endif
 	}
 	dabuf = malloc(XFS_DA_BUF_SIZE(nex));
 	if (dabuf == NULL) {
@@ -236,8 +247,12 @@ da_brelse(
 		bcopy(dabuf->bps, bplist, nbuf * sizeof(*bplist));
 	}
 	da_buf_done(dabuf);
-	for (i = 0; i < nbuf; i++)
+	for (i = 0; i < nbuf; i++) {
+#ifdef XR_PF_TRACE
+		pftrace("putbuf %p (%llu)", bplist[i], (long long)XFS_BUF_ADDR(bplist[i]));
+#endif
 		libxfs_putbuf(bplist[i]);
+	}
 	if (bplist != &bp)
 		free(bplist);
 }
@@ -853,7 +868,7 @@ process_sf_dir2(
 
 	sfp = &dip->di_u.di_dir2sf;
 	max_size = XFS_DFORK_DSIZE(dip, mp);
-	num_entries = INT_GET(sfp->hdr.count, ARCH_CONVERT);
+	num_entries = sfp->hdr.count;
 	ino_dir_size = INT_GET(dip->di_core.di_size, ARCH_CONVERT);
 	offset = XFS_DIR2_DATA_FIRST_OFFSET;
 	bad_offset = *repair = 0;
@@ -1440,13 +1455,6 @@ process_dir2_data(
 		} else if (INT_GET(dep->inumber, ARCH_CONVERT) == mp->m_sb.sb_gquotino) {
 			clearino = 1;
 			clearreason = _("group quota");
-		} else if (INT_GET(dep->inumber, ARCH_CONVERT) == old_orphanage_ino) {
-			/*
-			 * Do nothing, silently ignore it, entry has already
-			 * been marked TBD since old_orphanage_ino is set
-			 * non-zero.
-			 */
-			clearino = 0;
 		} else if ((irec_p = find_inode_rec(
 				XFS_INO_TO_AGNO(mp, INT_GET(dep->inumber,
 					ARCH_CONVERT)),
@@ -1993,9 +2001,6 @@ process_leaf_node_dir2(
 	int			t;
 	bmap_ext_t		lbmp;
 
-	if (do_prefetch)
-		prefetch_dir2(mp, blkmap);
-
 	*repair = *dot = *dotdot = good = 0;
 	*parent = NULLFSINO;
 	ndbno = NULLDFILOFF;
