@@ -148,7 +148,7 @@ EXTERN xfs_agblock_t	inobt_root;
 /* configuration vars -- fs geometry dependent */
 
 EXTERN int		inodes_per_block;
-EXTERN int		inodes_per_cluster;	/* inodes per inode buffer */
+EXTERN int		inodes_per_cluster;
 EXTERN unsigned int	glob_agcount;
 EXTERN int		chunks_pblock;	/* # of 64-ino chunks per allocation */
 EXTERN int		max_symlink_blocks;
@@ -166,12 +166,12 @@ EXTERN xfs_suminfo_t	*sumcompute;
 
 /* inode tree records have full or partial backptr fields ? */
 
-EXTERN int		full_backptrs;	/*
-					 * if 1, use backptrs_t component
-					 * of ino_un union, if 0, use
-					 * parent_list_t component.  see
-					 * incore.h for more details
-					 */
+EXTERN int		full_ino_ex_data;/*
+					  * if 1, use ino_ex_data_t component
+					  * of ino_un union, if 0, use
+					  * parent_list_t component.  see
+					  * incore.h for more details
+					  */
 
 #define ORPHANAGE	"lost+found"
 
@@ -182,9 +182,6 @@ EXTERN __uint64_t	sb_ifree;	/* free inodes */
 EXTERN __uint64_t	sb_fdblocks;	/* free data blocks */
 EXTERN __uint64_t	sb_frextents;	/* free realtime extents */
 
-EXTERN xfs_ino_t	orphanage_ino;
-EXTERN xfs_ino_t	old_orphanage_ino;
-
 /* superblock geometry info */
 
 EXTERN xfs_extlen_t	sb_inoalignmt;
@@ -195,9 +192,16 @@ extern size_t 		ts_dirbuf_size;
 extern size_t 		ts_dir_freemap_size;
 extern size_t 		ts_attr_freemap_size;
 
-EXTERN pthread_rwlock_t	*per_ag_lock;
+EXTERN pthread_mutex_t	*ag_locks;
 
-EXTERN int report_interval;
-EXTERN __uint64_t *prog_rpt_done;
+EXTERN int 		report_interval;
+EXTERN __uint64_t 	*prog_rpt_done;
+
+#ifdef XR_PF_TRACE
+EXTERN FILE		*pf_trace_file;
+#endif
+
+EXTERN int		ag_stride;
+EXTERN int		thread_count;
 
 #endif /* _XFS_REPAIR_GLOBAL_H */
